------------------------------------------------------------------------------
--                                                                          --
--                      gwindows.combo_boxes.ex_combo_boxes                 --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--                            $Revision: 1.5 $                              --
--                                                                          --
--                            Konad GmbH 2004                               --
--                                                                          --
-- This is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion. It is distributed in the hope that it will be useful,  but WITHOUT --
-- ANY WARRANTY;  without  even the  implied warranty of MERCHANTABILITY or --
-- FITNESS FOR A PARTICULAR PURPOSE.    See the GNU General  Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with this;  see file COPYING.  If not, write --
-- to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, --
-- MA 02111-1307, USA.                                                      --
--                                                                          --
-- As a special exception,  if other files  instantiate  generics from this --
-- unit, or you link  this unit with other files  to produce an executable, --
-- this  unit  does not  by itself cause  the resulting  executable  to  be --
-- covered  by the  GNU  General  Public  License.  This exception does not --
-- however invalidate  any other reasons why  the executable file  might be --
-- covered by the  GNU Public License.                                      --
--                                                                          --
-- More information about GWINDOWS and the most current public version can  --
-- be located on the web at http://www.gnavi.org/gwindows                   --
--                                                                          --
------------------------------------------------------------------------------
-- Features of Ex_Drop_Down_Combo_Box_Type:
--
-- The intended use is for database applications. The combo box is filled
-- with strings and associated id's (primary keys). The user may choose
-- very quick from the list if autocomplete is on. You may force the box
-- to only accept "valid" entries (which are in the associated list).
------------------------------------------------------------------------------
package Gwindows.Combo_Boxes.Ex_Combo_Boxes is
   type Ex_Drop_Down_Combo_Box_Type is new Drop_Down_Combo_Box_Type with private;

   -- adds an entry at the end (according to the sort-style)
   -- and returns the index of the added entry in Index
   procedure Add(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                 Value: in Gstring;
                 Index: out Natural);

   -- Necessary overwrite because of an error in Davids function
   --
   -- gwindows-combo_boxes.adb, line 532: "wparam: Natural" conflicts with
   -- default value zero for start_item
   function Find (Combo      : in ex_drop_down_combo_box_type;
                  Value      : in GString;
                  Start_Item : in Natural        := 0)
                 return Natural;

   function Find_Exact (Combo      : in ex_drop_down_combo_box_type;
                        Value      : in GString;
                        Start_Item : in Natural                     := 0)
                 return Natural;

   -- You may set and get an id for a list entry und use it in your
   -- application. Very handy for database filled lists (id=primary key)
   procedure Set_Item_Id(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                         Item: in natural;
                         Id: in Integer);
   function Get_Item_Id(Combo: in Ex_Drop_Down_Combo_Box_Type;
                        Item: in natural) return Integer;

   -- implements the Autocomplete behavior of the combo box
   procedure On_Edit_Change (Combo : in out Ex_Drop_Down_Combo_Box_Type);

   -- turn Autocomplete on and off
   procedure Set_Autocomplete(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                              Autocomplete: in Boolean);

   -- if Force=true the user may only enter "valid" strings (which are in the list)
   procedure Force_Valid_Text(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                              Force: in Boolean);

   -- set the height of the selection field in pixels
   procedure Set_Selection_Field_Height(Combo: in out Ex_Drop_Down_Combo_Box_Type;
                                        Height: in Natural);

private
   type Ex_Drop_Down_Combo_Box_Type is new Drop_Down_Combo_Box_Type with
      record
         Autocomplete: Boolean:= false;
         Force_Valid_text: Boolean:= false;
         Last_Text_Len: Natural := 0;
      end record;
end Gwindows.Combo_Boxes.Ex_Combo_Boxes;
