with Ada.Unchecked_Conversion; use Ada;
with Interfaces.C; use Interfaces.C;
with Gwindows.Gstrings; use Gwindows.Gstrings;
with Ada.Text_Io; use Ada;
with Gwindows.Types; use Gwindows.Types;
with Gwindows.Static_Controls;
with Gwindows.Edit_Boxes;

package body Gwindows.Windows.Ex_windows is

   WM_USER             :constant := 16#400#;
   TB_GETTOOLTIPS      :constant := WM_USER + 35;

   type NMTTDISPINFO_Type is
      record
         hdr       : Gwindows.Base.Notification;
         lptstr    : Interfaces.C.Int;
         SzText    : Char_Array(1..80);
         Hinst     : Interfaces.C.long;
         UFlags    : Interfaces.C.Int;
         Lparam    : Integer;
      end record;

   type NMTTDISPINFO_Pointer is access all NMTTDISPINFO_Type;

   function Message_To_nmttdispinfo_pointer is
      new Unchecked_Conversion(Pointer_To_Notification,
                               nmttdispinfo_pointer);

   procedure Translate_Key
     (wParam      : in     Interfaces.C.int;
      Special_Key :    out Special_Key_Type;
      Key         :    out GCharacter);

   ----------------------
   -- activate_tooltip --
   ----------------------

   procedure Activate_Tooltip(Window: in out Ex_Window_Type;
                              Toolbar: in out Ex_Toolbar_Control_Type)is

   begin
      Window.Toolbar := Toolbar'Unchecked_Access;
   end Activate_Tooltip;


   --------------------
   -- event-handling --
   --------------------

   ----------------
   -- on_message --
   ----------------

   procedure On_Message (Window       : in out Ex_Window_Type;
                         message      : in     Interfaces.C.unsigned;
                         wParam       : in     Interfaces.C.int;
                         lParam       : in     Interfaces.C.int;
                         Return_Value : in out Interfaces.C.long)is

      use Gwindows.Static_Controls;
      use Gwindows.Edit_Boxes;

      WM_KEYDOWN                 : constant := 256;
      WM_KEYUP                   : constant := 257;
      WM_CTLCOLOREDIT            : constant := 16#0133#;
      WM_ERASEBKGND              : constant := 20;
      WM_CTLCOLORSTATIC          : constant := 312;
      WM_CTLCOLORBTN             : constant := 16#0135#;

      Area   : Rectangle_Type := (Top => 0, Left => 0,
                                  Bottom => Client_Area_Height(Window),
                                  Right => Client_Area_Width(Window));
      Control : Pointer_To_Base_Window_Class :=
        Window_From_Handle (long (lParam));
      Canvas : Canvas_Type;
   begin
      case Message is
         when WM_ERASEBKGND =>
            Handle (Window.Canvas, GWindows.Types.Handle (wParam));
            Fill_Rectangle(Window.Canvas, Area, Window.Brush);

         when WM_CTLCOLORSTATIC =>
            if Control /= null then
               if Control.all in Label_Type'Class then
                  Handle (Canvas, GWindows.Types.Handle (wParam));
                  Background_Color (Canvas, Ex_Window_Type(Parent(Control.all).all).Bk_color);
                  Handle (Canvas, 0);
                  Return_Value := Handle (Ex_Window_Type(Parent(Control.all).all).Brush);
               elsif Control.all in Edit_box_Type'Class then
                  Handle (Canvas, GWindows.Types.Handle (wParam));
                  Background_Color (Canvas, Ex_Window_Type(Parent(Control.all).all).Bk_color);
                  Handle (Canvas, 0);
                  Return_Value := Handle (Ex_Window_Type(Parent(Control.all).all).Brush);
               end if;
            end if;
         when WM_CTLCOLORBTN =>
            null;
         when WM_KEYDOWN =>
            declare
               Special : Special_Key_Type;
               Key     : GCharacter;
            begin
               Translate_Key (wParam, Special, Key);
               On_Character_Down (Window_Type'Class (Window),
                                  Special,
                                  Key);
               Return_Value := 0;
            end;

         when WM_KEYUP =>
            declare
               Special : Special_Key_Type;
               Key     : GCharacter;
            begin
               Translate_Key (wParam, Special, Key);
               On_Character_Up (Window_Type'Class (Window),
                                Special,
                                Key);
               Return_Value := 0;
            end;
         when others =>
            On_Message(Window_Type(Window), Message, Wparam, Lparam, Return_Value);
      end case;
   end On_Message;

   ---------------
   -- on_notify --
   ---------------

   procedure On_Notify (Window       : in out Ex_Window_Type;
                        Message      : in     Pointer_To_Notification;
                        Control      : in     Pointer_To_Base_Window_Class;
                        Return_Value : in out Interfaces.C.long) is

      Dispinfo_Ptr  : NMTTDISPINFO_Pointer := Message_To_Nmttdispinfo_Pointer(Message);
      Last : Size_t;

   begin
      case Message.Code is
         when -520 =>
            To_C(Item => To_String(Get_Tooltip(Window.Toolbar.all, Dispinfo_Ptr.Hdr.Id)),
                 Target => Dispinfo_Ptr.SzText, count => last);
      when Others =>
         On_Notify(Window_Type(Window), Message, Control, Return_Value);
      end case;

   exception
      when others =>
         null;
   end On_notify;
   ------------------------------------------------------------------------------------
   procedure Translate_Key
     (wParam      : in     Interfaces.C.int;
      Special_Key :    out Special_Key_Type;
      Key         :    out GCharacter)
   is
      use Interfaces.C;

      type State_Array is array (0 .. 255) of Character;
      Keyboard_State : State_Array := (others => ' ');

      procedure GetKeyboardState
        (lpbKeyState : State_Array := Keyboard_State);
      pragma Import (StdCall, GetKeyboardState, "GetKeyboardState");

      Key_Value : aliased Interfaces.C.unsigned := 0;

      procedure ToAscii
        (uVirtKey    : in     Interfaces.C.int      := wParam;
         uScanCode   : in     Interfaces.C.int      := 0;
         lpbKeyState : in     State_Array           := Keyboard_State;
         lpChar      : access Interfaces.C.unsigned := Key_Value'Access;
         uFlags      : in     Interfaces.C.int      := 0);
      pragma Import (StdCall, ToAscii, "ToAscii");

      procedure ToUnicode
        (uVirtKey    : in     Interfaces.C.int      := wParam;
         uScanCode   : in     Interfaces.C.int      := 0;
         lpbKeyState : in     State_Array           := Keyboard_State;
         lpChar      : access Interfaces.C.unsigned := Key_Value'Access;
         ccBuff      : in     Integer               := 1;
         uFlags      : in     Interfaces.C.int      := 0);
      pragma Import (StdCall, ToUnicode, "ToUnicode");

      VK_SHIFT                   : constant := 16;
      VK_ESCAPE                  : constant := 27;
      VK_PAUSE                   : constant := 19;
      VK_CAPITAL                 : constant := 20;
      VK_PRIOR                   : constant := 33;
      VK_NEXT                    : constant := 34;
      VK_END                     : constant := 35;
      VK_HOME                    : constant := 36;
      VK_LEFT                    : constant := 37;
      VK_UP                      : constant := 38;
      VK_RIGHT                   : constant := 39;
      VK_DOWN                    : constant := 40;
      VK_SNAPSHOT                : constant := 44;
      VK_INSERT                  : constant := 45;
      VK_DELETE                  : constant := 46;
      VK_F1                      : constant := 112;
      VK_F2                      : constant := 113;
      VK_F3                      : constant := 114;
      VK_F4                      : constant := 115;
      VK_F5                      : constant := 116;
      VK_F6                      : constant := 117;
      VK_F7                      : constant := 118;
      VK_F8                      : constant := 119;
      VK_F9                      : constant := 120;
      VK_F10                     : constant := 121;
      VK_F11                     : constant := 122;
      VK_F12                     : constant := 123;
      VK_NUMLOCK                 : constant := 144;
      VK_SCROLL                  : constant := 145;
   begin
      Key := GCharacter'Val (0);
      case wParam is
         when VK_ESCAPE =>
            Special_Key := Escape;
         when VK_PAUSE =>
            Special_Key := Pause;
         -- is not processed because we route Shift to Caps_Lock
         --when VK_CAPITAL =>
         --   Special_Key := Caps_Lock;
         when VK_PRIOR =>
            Special_Key := Page_Up;
         when VK_NEXT =>
            Special_Key := Page_Down;
         when VK_END =>
            Special_Key := End_Key;
         when VK_HOME =>
            Special_Key := Home_Key;
         when VK_LEFT =>
            Special_Key := Left_Key;
         when VK_UP =>
            Special_Key := Up_Key;
         when VK_RIGHT =>
            Special_Key := Right_Key;
         when VK_DOWN =>
            Special_Key := Down_Key;
         when VK_SNAPSHOT =>
            Special_Key := Print_Screen;
         when VK_INSERT =>
            Special_Key := Insert;
         when VK_DELETE =>
            Special_Key := Delete;
         when VK_F1 =>
            Special_Key := F1;
         when VK_F2 =>
            Special_Key := F2;
         when VK_F3 =>
            Special_Key := F3;
         when VK_F4 =>
            Special_Key := F4;
         when VK_F5 =>
            Special_Key := F5;
         when VK_F6 =>
            Special_Key := F6;
         when VK_F7 =>
            Special_Key := F7;
         when VK_F8 =>
            Special_Key := F8;
         when VK_F9 =>
            Special_Key := F9;
         when VK_F10 =>
            Special_Key := F10;
         when VK_F11 =>
            Special_Key := F11;
         when VK_F12 =>
            Special_Key := F12;
         when VK_NUMLOCK =>
            Special_Key := Number_Lock;
         when VK_SCROLL =>
            Special_Key := Scroll_Lock;
            -- Shift is routed to Caps_Lock because Davids Special_Key_Type
            -- does not contain the Shift-Key
         when VK_SHIFT =>
            Special_Key := Caps_Lock;
         when others =>
            Special_Key := None;
            GetKeyboardState;
            if Character_Mode = Unicode then
               ToUnicode;
            else
               ToAscii;
            end if;

            if Key_Value > 255 then
               Key_Value := Key_Value - (Key_Value and 16#FF00#);
            end if;
            Key := GCharacter'Val (Key_Value);
      end case;

   end Translate_Key;

   procedure On_Create (Window : in out ex_Window_Type) is
   begin
      Window.Bk_Color := System_Color(COLOR_BTNFACE);
      Create_Solid_Brush(Window.Brush, Window.Bk_color);
      On_Create(Window_Type(Window));
   end On_Create;

   procedure Set_Bk_Color(Window: in out ex_Window_Type;
                          Color : in Color_type) is
   begin
      Window.Bk_Color := Color;
      Create_Solid_Brush(Window.Brush, Window.Bk_Color);
      Redraw(Window, True);
   end Set_Bk_Color;

   function Get_Bk_Color(Window: in ex_Window_Type) return Color_Type is
   begin
      return Window.Bk_Color;
   end Get_Bk_Color;

end Gwindows.Windows.Ex_windows;
